
/* COUNTLIB.C v2.4 */

#include "countlib.h"


static int mode = 0;
static char graphpath[MAX_PATHNAME+1] = DEFAULT_GRAPHPATH;
static char filename[MAX_PATHNAME+1] = DEFAULT_FILENAME;
static char logfilename[MAX_PATHNAME+1] = DEFAULT_LOGNAME;
static char entryname[MAX_ENTRYNAME+1] = "";
static int out_ndigits = N_DIGITS;
static int logmode = 0;
static char s_width[MAX_NUMS+1] = DEFAULT_WIDTH;
static char s_height[MAX_NUMS+1] = DEFAULT_HEIGHT;


static char *my_itoa(int n, char *s)

   {int i = 0;
    int j, l;
    char ch;
    while (n > 0)
       {s[i] = '0' + (n % 10);
        n /= 10;
        i++;}
    s[i] = '\0';
    l = i;
    for(i = 0, j = l-1; i < l/2; i++, j--)
       {ch = s[i];
        s[i] = s[j];
        s[j] = ch;}
    return s;
   }


static void AdjustCase(void)

   {int i;
    int low = 0;
    if (mode & 16)
        for(i=0; i<strlen(entryname); i++)
            entryname[i] = tolower(entryname[i]);
    else
        for(i=0; i<strlen(entryname); i++)
           {if (entryname[i] == USER_CHAR)
	        low = 1;
	    else if (entryname[i] == PATH_CHAR)
	        low = 0;
	    if (low)
	        entryname[i] = tolower(entryname[i]);}
   }


static void LogWrite(char *s)

   {FILE *fp;
    fp = fopen(logfilename, "a");
    if (fp != NULL)
       {fprintf(fp, "%s", s);
        fclose(fp);}
   }


static void LogWriteln(char *s)

   {FILE *fp;
    fp = fopen(logfilename, "a");
    if (fp != NULL)
       {fprintf(fp, "%s\n", s);
        fclose(fp);}
   }


static void LogWriteInt(int n)

   {FILE *fp;
    fp = fopen(logfilename, "a");
    if (fp != NULL)
       {fprintf(fp, "%d\n", n);
        fclose(fp);}
   }


static void MakeZeroedString(char *s, long int v)

   {int k = 1;
    s[N_DIGITS] = '\0';
    while ((v > 0) && (k <= N_DIGITS))
       {s[N_DIGITS-k] = v%10 + '0';
	v /= 10;
	k++;}
    k = N_DIGITS-k;
    for(;k >= 0; k--)
	s[k] = '0';
   }


static void GraphOutput(char *zs)

   {int i;
    if (mode & 8)
        i = N_DIGITS - out_ndigits;
    else
       {i = 0;
        while (zs[i] == '0') i++;}
    for(; i < N_DIGITS; i++)
       {printf("<IMG SRC=\"%s%s%c%s\" ALT=\"%c\"", graphpath, NUMPREF, zs[i], NUMSUFF, zs[i]);
        if (mode & 64)
            printf(" WIDTH=\"%s\"", s_width);
        if (mode & 128)
            printf(" HEIGHT=\"%s\"", s_height);
        printf(" >");}
   }


static void TextOutput(char *zs, long int v)

   {int i;
    if (mode & 8)
        for(i = N_DIGITS - out_ndigits; i < N_DIGITS; i++)
	    printf("%c", zs[i]);
    else
        printf("%ld", v);
   }


static void UpdateCounter(void)

   {char sbuf[MAX_ENTRYNAME];
    char zs[N_DIGITS+1];
    FILE *fp;
    fpos_t pos;
    long int v;
    sbuf[0] = '\0';
    if (!(mode & 32))
        AdjustCase();
    if (logmode)
       {LogWrite("Using entryname ");
        LogWriteln(entryname);}
    fp = fopen(filename, "r+");
    if (fp == NULL)
       {fp = fopen(filename, "w");
        if (fp != NULL)
           {if (logmode)
               {LogWrite("Creating data file previously not found: ");
                LogWriteln(filename);}
            v = ((mode & 2) ? 0 : 1);
            MakeZeroedString(zs, v);
            fprintf(fp, "%s %s\n", entryname, zs);
	    fclose(fp);
            if (!(mode & 1))
                if (mode & 4)
                    GraphOutput(zs);
                else
                    TextOutput(zs, v);
           }
        else
           {fclose(fp);
   	    printf("%s", EMSG_FILENOTFOUND);
            if (logmode)
               {LogWrite("ERROR - Could not find or create data file: ");
                LogWriteln(filename);}
          }
       } 
    else
       {if (logmode)
	   {LogWrite("Opened data file: ");
            LogWriteln(filename);}
        while ((!(feof(fp))) && (strcmp(sbuf, entryname)))
	   {fgetpos(fp, &pos);
	    fscanf(fp, "%s %ld\n", sbuf, &v);}
	if (!(strcmp(sbuf, entryname)))
	    fsetpos(fp, &pos);
        else
	   {fseek(fp, 0L, SEEK_END);
            v = 0;}
	if (!(mode & 2)) v++;
        MakeZeroedString(zs, v);
        fprintf(fp, "%s %s\n", entryname, zs);
	fclose(fp);
        if (!(mode & 1))
            if (mode & 4)
                GraphOutput(zs);
            else
                TextOutput(zs, v);
       }
   if (logmode)
        LogWrite("Exiting...\n\n");
   }


static void ReadSubArgument(char *dest, char *argsrc, int destlen)

   {int j = 1;
    do
        {j++;
	 dest[j-2] = argsrc[j];}
    while ((argsrc[j] != '\0') && (j < destlen));
    dest[destlen] = '\0';
   }


static void AddPathChar(char *dest)

   {int pos;
    pos = strlen(dest)-1;
    if (dest[pos] != PATH_CHAR)
       {pos++;
        dest[pos++] = PATH_CHAR;
        dest[pos] = '\0';}
   }


static void ProcessArguments(int argc, char **argv)

   {int i;
    if (argc > 1)
       {for (i = 1; i < argc; i++)
           switch (argv[i][1])
	      {case 'q': mode++;
	                 break;
	       case 'r': mode += 2;
	                 break;
               case 'g': mode += 4;
                         if (argv[i][2] != '\0')
                             ReadSubArgument(graphpath, argv[i], MAX_PATHNAME);
                         break;
               case 'n': if (argv[i][2] != '\0')
                             ReadSubArgument(entryname, argv[i], MAX_ENTRYNAME);
                         break;
               case 'f': if (argv[i][2] != '\0')
                             ReadSubArgument(filename, argv[i], MAX_PATHNAME);
                         break;
	       case 'd': mode += 8;
                         if (argv[i][2] != '\0')
                             out_ndigits = argv[i][2] - '0';
	                 break;
               case 'l': logmode = 1;
	                 if (argv[i][2] != '\0')
                             ReadSubArgument(logfilename, argv[i], MAX_PATHNAME);
                         break;
               case 'c': switch (argv[i][2])
	                    {case '0': mode += 16; break;
                             case '2': mode += 32; break;}
                         break;
               case 'w': mode += 64;
	                 if (argv[i][2] != '\0')
                             ReadSubArgument(s_width, argv[i], MAX_NUMS);
                         break;
               case 'h': mode += 128;
	                 if (argv[i][2] != '\0')
                             ReadSubArgument(s_height, argv[i], MAX_NUMS);
                         break;
              }
       }
   }


static void CountMain(void)

   {char *p;
    AddPathChar(graphpath);
    if (logmode)
       {LogWrite("VBCount v2.4: arguments processed. Starting execution with mode ");
        LogWriteInt(mode);}
    if (entryname[0] == '\0')
       {p = getenv("DOCUMENT_URI");
        if (p != NULL)
           {if (logmode)
	       {LogWrite("No entryname supplied, got DOCUMENT_URI: ");
                LogWriteln(p);}
	    strcpy(entryname, p);
            UpdateCounter();}
        else
           {if (logmode) LogWriteln("ERROR - No entryname supplied, no DOCUMENT_URI found.");
	    printf("%s\n", EMSG_NO_DOCURI);}
       }
    else
       {if (logmode)
            LogWriteln("Entryname supplied on the command line.");
        UpdateCounter();}
   }


void Count(int argc, char **argv)

   {ProcessArguments(argc, argv);
    CountMain();
   }


void DoCount(char *ename, char *fname, int a_mode)

   {if (ename != NULL)
        strcpy(entryname, ename);
    if (fname != NULL)
        strcpy(filename, fname);
    if ((a_mode > 0) && (a_mode <= MAX_MODE))
        mode = a_mode;
    CountMain();
   }


void SetGraphicCountParams(char *gpath, int w, int h)

   {if (gpath != NULL)
        strcpy(graphpath, gpath);
    if ((w > 0) && (w < MAX_NUMS_NUMBER))
        my_itoa(w, s_width);
    if ((h > 0) && (h < MAX_NUMS_NUMBER))
        my_itoa(h, s_height);
   }


void SetAdvancedCountParams(char *lname, int a_logmode, int a_ndigits)

   {if (lname != NULL)
        strcpy(logfilename, lname);
    if (a_logmode)
        logmode = 1;
    if ((a_ndigits > 0) && (a_ndigits <= N_DIGITS))
        out_ndigits = a_ndigits;
   }
